from kubernetes import client, config, utils
import yaml


def get_api_client():
    config.load_kube_config()
    return client.ApiClient()


def create_resources_from_yaml(manifest):
    yml_document_all = yaml.safe_load_all(manifest)
    for yml_document in yml_document_all:
        if yml_document: # Calico YAML에서 비어있는 YAML 항목이 None으로 들어오고 있으므로 주의
            utils.create_from_dict(get_api_client(), yml_document)



import json
import logging

from flask import Flask, request

from master.server.join import join as worker_join

app = Flask(__name__)


def run(host, port):
    app.run(debug=True, host=host, port=port, use_reloader=False)


@app.route('/hello', methods=['get'])
def hello():
    data = request.get_json()
    logging.info(data)

    response = app.response_class(
        response=json.dumps({}),
        status=200,
        mimetype='application/json'
    )
    return response


@app.route('/join', methods=['post'])
def join():
    join_data = worker_join()
    response = app.response_class(
        response=json.dumps(join_data),
        status=200,
        mimetype='application/json'
    )
    return response



import os
import logging
import subprocess

import requests

from master import utils
from master.k8s import k8s

CALICO_YAML_URL = 'https://docs.projectcalico.org/v3.8/manifests/calico.yaml'


def setup(nic):
    logging.info("[Setup]\n")

    # get configuration
    if nic == None:
        iface = utils.get_default_iface()
    else:
        iface = nic
    token = utils.get_join_token()
    kube_version, cni_version, containerd_version = utils.get_compatible_versions()

    # Run install-master script from internet
    env = os.environ
    env = dict({
        "ADVERTISE_NET_DEV": iface,
        "JOIN_TOKEN": token,
        "KUBERNETES_VERSION": kube_version,
        "KUBERNETES_CNI_VERSION": cni_version,
        "CONTAINERD_VERSION": containerd_version
    }, **env)
    command = "cd /tmp && curl -s https://raw.githubusercontent.com/AI-Ocean/kubernetes-install-scripts/main/install-master.sh | bash"

    process = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, env=env)

    # print progress
    while True:
        output = process.stdout.readline()
        if output == b'' and process.poll() is not None:
            break
        if output:
            print(output.strip().decode())

    return True


def setup_calico(pod_cidr):
    logging.info('Installing calico...')
    response = requests.get(url=CALICO_YAML_URL)

    # 별도의 pod cidr을 명시한 경우에 replace 해준다.
    manifest = response.text.replace('192.168.0.0/16', pod_cidr) if pod_cidr else response.text
    k8s.create_resources_from_yaml(manifest)

    logging.info('Completed install calico.')


def setup_system_service():
    response = requests.get(url='https://raw.githubusercontent.com/AI-Ocean/kubernetes-install-scripts/main/ocean-agent-master.service')
    with open("/lib/systemd/system/ocean-agent-master.service", "w") as f:
        f.write(response.text)

    subprocess.check_output(['systemctl', 'daemon-reload'])
    subprocess.check_output(['systemctl', 'enable', 'ocean-agent-master'])
    subprocess.check_output(['systemctl', 'restart', 'ocean-agent-master'])



import logging

from master import utils


def join():
    logging.info("[Join]")

    # send join info
    token = utils.get_join_token(ttl_hash=utils.get_ttl_hash())
    kube_version, cni_version, containerd_version = utils.get_compatible_versions()

    join_info = {
        "JOIN_TOKEN": token,
        "KUBERNETES_VERSION": kube_version,
        "KUBERNETES_CNI_VERSION": cni_version,
        "CONTAINERD_VERSION": containerd_version
    }

    return join_info



import os
import random
import string
import subprocess
from functools import lru_cache
import time
import urllib3
import logging

import netifaces as ni
import requests
import dns.resolver


urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


def get_default_iface():
    default_iface = [iface for iface in os.listdir('/sys/class/net/') if iface != 'lo' and iface != 'docker0'][0]
    return default_iface


@lru_cache()
def get_join_token(ttl_hash=None):
    if check_kubeadm_initialized():
        return str(subprocess.check_output(['kubeadm', 'token', 'create']).decode("utf-8").strip())
    else:
        return f'{get_random_string(6)}.{get_random_string(16)}'


def get_ttl_hash(seconds=1800):
    return round(time.time() / seconds)


# 바꾸지 않아도 문제는 없으니 일단 이대로 놔둔다.
def get_compatible_versions():
    kube_version = "1.20.8-00"
    cni_version = "0.8.7-00"
    containerd_version = "1.5.2-0ubuntu1~18.04.2"

    return kube_version, cni_version, containerd_version


# default iface의 1번째 IP만을 반환한다. NIC에 IP가 여러개 붙어있는 경우는 고려하지 않음.
def get_api_server_address(nic):
    if nic:
        return ni.ifaddresses(nic)[ni.AF_INET][0]['addr']
    else:
        return ni.ifaddresses(get_default_iface())[ni.AF_INET][0]['addr']


def get_random_string(length):
    return ''.join(random.choices(string.ascii_lowercase + string.digits, k=length))


def check_kubeadm_initialized():
    try:
        response = requests.get('https://localhost:6443', verify=False)
        if response.status_code == 403: # If api server is running, 403 should be returned
            print('Kubernetes API server is already running')
            return True
    except requests.exceptions.ConnectionError:
        return False


def set_host_from_kubernetes_dns(host, host_fqdn):
    resolver = dns.resolver.Resolver()
    resolver.nameservers = ['10.96.0.10'] # 쿠버 IP는 고정
    answers = resolver.resolve(host_fqdn, "A")
    ip = answers[0].to_text()
    record = "{} {}\n".format(ip, host)

    path = '/etc/hosts'
    lines = open(path, 'r').readlines()

    found = False
    changed = False

    for i, v in enumerate(lines):
        if host in v:
            found = True
            if v.split(' ')[0] != ip:
                lines[i] = record
                changed = True
            break

    if changed or not found:
        if not found:
            lines.append(record)
            logging.info("New record : {}".format(host))
        out = open(path, 'w')
        out.writelines(lines)
        out.close()
        logging.info("Synced record : {}".format(host))
